#pragma once

#ifdef LIBTERAVIEW_EXPORTS
#define LIBTERAVIEW_API __declspec(dllexport)
#else
#define LIBTERAVIEW_API __declspec(dllimport)
#endif

#include <stdio.h>





#ifdef __cplusplus    // If used by C++ code, 
extern "C" {          // we need to export the C interface
#endif




LIBTERAVIEW_API int sizeX, sizeY, dataLength;
LIBTERAVIEW_API char deviceIDstring[32];

typedef  signed __int16 TS_ELEMENT;
typedef  TS_ELEMENT *   TS_DATA;


#define TS_MAX_VALUE 1023
#define TS_RAW_LIMIT 700

#define DEFAULT_EXPOSURE 3   //Between 0 and 10
#define DEFAULT_THRESHOLD 20


typedef  int    TS_RES;
#define TS_OK 0
#define TS_FAILED -9999
#define TS_CANCELLED -9998
#define TS_ERR_WRONG_PARAMETERS -9997
#define TS_NOTRUNNING -9996
#define TS_ERR_SIZE_MISMATCH -9995
#define TS_OUTPUT_ERROR -9994
#define TS_INPUT_ERROR -9993
#define TS_NOTCONFIGURED -9992
#define TS_PATHNOTFOUND -9991


//Initialize device
LIBTERAVIEW_API TS_RES tsInit(void);

//Close the device
LIBTERAVIEW_API TS_RES tsClose(void);

//Starts data acquisition and processing
LIBTERAVIEW_API TS_RES tsStart(void);


//Stops data acquisition and processing
LIBTERAVIEW_API TS_RES tsStop(void);

//Checks whether the acquisition is running
LIBTERAVIEW_API int tsIsRunning(void);

//Read processed data to buffer (acquistion must be running). 
LIBTERAVIEW_API TS_RES tsRead(TS_DATA buffer);

//Read unprocessed data to buffer (acquistion must be running).
LIBTERAVIEW_API TS_RES tsReadRaw(TS_DATA buffer);

// Sets current exposure.
LIBTERAVIEW_API TS_RES tsSetExposure(int exposure);
// Gets current exposure.
LIBTERAVIEW_API int tsGetExposure(void);

// Gets admissible range of exposures
LIBTERAVIEW_API TS_RES tsGetExposureRange(int * pMin, int * pMax);

// Gets length of the integration time for the given exposure; if parameter is out of range (i.e., negative), data for the current exposure is returned
LIBTERAVIEW_API double tsGetIntTime(int exposure);

// Sets threshold value
LIBTERAVIEW_API TS_RES tsSetThreshold(double threshold);
// Gets threshold value
LIBTERAVIEW_API double tsGetThreshold(void);

// Turns on/off difference mode
LIBTERAVIEW_API TS_RES tsSetDifference(int on);

// Turns on/off greedy stitch mode
LIBTERAVIEW_API TS_RES tsSetGreedy(int on);

// Format of the config file: header = <magic string (6 bytes)><dataLength (2 bytes)><norm list size (1 byte)><bg list size (1 byte)>
// main body  = <normalizations (2*norm list size*2*dataLength)><backgrounds (bg list size*2*dataLength)>
LIBTERAVIEW_API TS_RES tsLoadConfig(FILE *stream);
LIBTERAVIEW_API TS_RES tsSaveConfig( FILE *stream);


// Helper functions.  The argument containing path to the config file is ANSI string for tsLoadConfigFrom(), while it is a wide-character string for tsLoadConfigFrom_w().
// If the file "filename" cannot be opened for any reason, returns TS_PATHNOTFOUND.
LIBTERAVIEW_API TS_RES tsLoadConfigFrom(const char *filename);
LIBTERAVIEW_API TS_RES tsLoadConfigFrom_w(const wchar_t *filename);


// Helper functions.  The argument containing path to the config file is ANSI string for tsSaveConfigAs(), while it is a wide-character string for tsSaveConfigAs_w().
// If the file already exists, it will be overwritten silently. If the file "filename" cannot be opened for any reason, returns TS_PATHNOTFOUND.
LIBTERAVIEW_API TS_RES tsSaveConfigAs(const char *filename);
LIBTERAVIEW_API TS_RES tsSaveConfigAs_w(const wchar_t *filename);

// Ticker callback function. Completion parameter varies from 0 (just started) to 1 (finished). The function should return TRUE to request abort of the operation.
typedef  int (*ts_ticker_t)(double completion);

// Record background data
LIBTERAVIEW_API TS_RES tsRecordBackground(ts_ticker_t callback);

// Record normalization data
LIBTERAVIEW_API TS_RES tsRecordNormalization(ts_ticker_t callback);

//Select nomaliztion (0 - default, 1 - recorded)
LIBTERAVIEW_API TS_RES tsSelectNorm(int i);



LIBTERAVIEW_API int tsSizeX(void);
LIBTERAVIEW_API int tsSizeY(void); 
LIBTERAVIEW_API int tsDataLength(void);
LIBTERAVIEW_API char* tsDeviceID(void);


#ifdef __cplusplus
}
#endif